import bpy

from bpy.types import Operator

from mathutils import Vector


class UW_OT_CreateSpawnPoint(Operator):
    bl_idname = "uw_interactions.add_create_spawn_point"
    bl_label = "Add Spawn Point"
    bl_description = "Add a spawn point at the 3d cursor."

    @classmethod
    def poll(cls, context):
        obj = context.selected_objects
        if len(obj) == 0:
            return True

        return False

    def get_or_create_collection(self, name: str):
        collection = bpy.data.collections.get(name)

        if not(collection):
            collection = bpy.data.collections.new(name)
            bpy.context.scene.collection.children.link(collection)

        return collection

    def execute(self, context):

        # SCENARIO COLLECTION
        scenariosCollection = self.get_or_create_collection('Scenarios')

        default_scenario = scenariosCollection.objects.get("default")
        if default_scenario != None:
            bpy.data.objects.remove(default_scenario, do_unlink=True)

        # DEFAULT SPHERE
        default_sphere = bpy.data.objects.new("default", None)
        scenariosCollection.objects.link(default_sphere)

        default_sphere.empty_display_size = 2
        default_sphere.empty_display_type = 'SPHERE'
        default_sphere.location = bpy.context.scene.cursor.location + \
            Vector((0, 0, 10))
            
        
        default_sphere["data"] = "scenario"
        default_sphere["camera_angle"] = 0
        default_sphere["default"] = "true"
        default_sphere["desc_content"] = "Explore the world!"
        default_sphere["desc_title"] = "Default spawn"
        default_sphere["name"]="Free roam"
        
        
            
        
        # DEFAULT SPAWN POINT
        default_spawn = bpy.data.objects.new("Spawn.001", None)
        scenariosCollection.objects.link(default_spawn)
        
        default_spawn["data"] = "spawn"
        default_spawn["type"] = "player"


        default_spawn.empty_display_size = 2
        default_spawn.empty_display_type = 'ARROWS'
        default_spawn.location = bpy.context.scene.cursor.location - default_sphere.location
        print(default_spawn.location)
        default_spawn.parent = default_sphere
        print(default_spawn.location)
        

        return {'FINISHED'}
