import bpy

from bpy.types import Operator

class UW_OT_AddOceanMaterialToActive(Operator):
    bl_idname = "uw_material.add_ocean_material_to_object"
    bl_label = "Add Ocean Material"
    bl_description = "Create or add ocean material to current object"
    
    
    @classmethod
    def poll(cls,context):
        obj = context.object
        
        if obj is not None:
            if obj.mode== "OBJECT":
                return True
            
        return False
    

    def execute(self,context):
        ob = bpy.context.active_object
        bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)

        # Get material
        mat = bpy.data.materials.get("ocean")
        if mat is None:
            # create material
            mat = bpy.data.materials.new(name="ocean")
            
        # Enable nodes
        mat.use_nodes = True
        
        nodes = mat.node_tree.nodes
        links = mat.node_tree.links
        
        nodes.clear()
        
        materialOutput = nodes.new("ShaderNodeOutputMaterial")
        materialOutput.location = (300,0)
        
        bsdf = nodes.new("ShaderNodeBsdfPrincipled")
        
        links.new(bsdf.outputs["BSDF"],materialOutput.inputs["Surface"])
        
        
        bsdf = mat.node_tree.nodes["Principled BSDF"]
        
        # Set material props
        bsdf.subsurface_method = 'BURLEY'
        bsdf.inputs['Base Color'].default_value = (0,0,0,1)
        bsdf.inputs['Metallic'].default_value = 0.0
        bsdf.inputs['Specular'].default_value = 0.5
        bsdf.inputs['Specular Tint'].default_value = 0.0
        bsdf.inputs['Roughness'].default_value = 0.0
        bsdf.inputs['Anisotropic'].default_value = 0.0
        bsdf.inputs['Anisotropic Rotation'].default_value = 0.0
        bsdf.inputs['Sheen'].default_value = 0.0
        bsdf.inputs['Sheen Tint'].default_value = 0.5
        bsdf.inputs['Clearcoat'].default_value = 1.0
        bsdf.inputs['Clearcoat Roughness'].default_value = 0.170
        bsdf.inputs['IOR'].default_value = 1.45
        bsdf.inputs['Transmission'].default_value = 1.00
        bsdf.inputs['Transmission Roughness'].default_value = 0.00
        bsdf.inputs['Emission'].default_value = (0,0,0,1)
        bsdf.inputs['Emission Strength'].default_value = 1.0
        bsdf.inputs['Alpha'].default_value = 1.0
        
        

        bumpNode = nodes.new("ShaderNodeBump")
        bumpNode.location = (-300,-500)
        
        bumpNode.inputs["Strength"].default_value = 1.0
        
        links.new( bumpNode.outputs['Normal'],nodes["Principled BSDF"].inputs["Normal"])
        
        texMusGrave1 = nodes.new("ShaderNodeTexMusgrave")
        texMusGrave1.location = (-800,-300)
        texMusGrave1.musgrave_dimensions = '3D'
        texMusGrave1.musgrave_type = 'MULTIFRACTAL'
        texMusGrave1.inputs["Scale"].default_value = 2.0
        texMusGrave1.inputs["Detail"].default_value = 0.05
        texMusGrave1.inputs["Dimension"].default_value = 2.0
        texMusGrave1.inputs["Lacunarity"].default_value = 1.0
        
        
        texMusGrave2 = nodes.new("ShaderNodeTexMusgrave")
        texMusGrave2.location = (-800,-600)
        texMusGrave2.musgrave_dimensions = '3D'
        texMusGrave2.musgrave_type = 'MULTIFRACTAL'
        texMusGrave2.inputs["Scale"].default_value = 0.5
        texMusGrave2.inputs["Detail"].default_value = 0.1
        texMusGrave2.inputs["Dimension"].default_value = 2.0
        texMusGrave2.inputs["Lacunarity"].default_value = 1.0
        
        mathNode = nodes.new("ShaderNodeMath")
        mathNode.location = (-500,-500)
        
        links.new(mathNode.outputs["Value"],bumpNode.inputs["Height"])
        
        links.new(texMusGrave1.outputs[0], mathNode.inputs[0])
        links.new(texMusGrave2.outputs[0], mathNode.inputs[1])

        vectorDisplacement = nodes.new("ShaderNodeVectorDisplacement")
        vectorDisplacement.location = (-1000,-600)
        vectorDisplacement.inputs["Midlevel"].default_value = 0.394
        vectorDisplacement.inputs["Scale"].default_value = 1
        vectorDisplacement.space = 'OBJECT'

        
        links.new(vectorDisplacement.outputs[0], texMusGrave1.inputs["Vector"])
        links.new(vectorDisplacement.outputs[0], texMusGrave2.inputs["Vector"])
        
        geometry = nodes.new("ShaderNodeNewGeometry")
        geometry.location = (-1200,-600)
        links.new(geometry.outputs[0], vectorDisplacement.inputs["Vector"])

        # Assign it to object
        if ob.data.materials:
            # assign to 1st material slot
            ob.data.materials[0] = mat
        else:
            # no slots
            ob.data.materials.append(mat)
            
        return {'FINISHED'}