"""
Operator to add physics for one or more objects.

Author: Vivasvan
Company: UltimateWorld
"""
import bpy
import bmesh
from mathutils import Vector


from bpy.types import Operator


class UW_AddPhysicsToSelectedObject(Operator):
    bl_idname = "uw_physics.add_physics_to_selected_object"
    bl_label = "Add physics"
    bl_description = "Add Physics for UltimateWorld to selected object"

    @classmethod
    def poll(cls, context):
        obj = context.object

        if obj is not None:
            if obj.mode == "OBJECT":
                return True

        return False

    def get_or_create_collection(self, name: str):
        collection = bpy.data.collections.get(name)

        if not(collection):
            collection = bpy.data.collections.new(name)
            bpy.context.scene.collection.children.link(collection)

        return collection

    def find_center(self, o):
        vcos = [o.matrix_world * v.co for v in o.data.vertices]
        def findCenter(l): return (max(l) + min(l)) / 2

        x, y, z = [[v[i] for v in vcos] for i in range(3)]
        center = [findCenter(axis) for axis in [x, y, z]]

    def execute(self, context):
        ob = bpy.context.active_object
        if ob == None:
            ob = bpy.context.object

        # setting the origin to center of the bounding box
        bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='BOUNDS')

        scene = bpy.context.scene
        bpy.ops.object.transform_apply(
            location=False, rotation=False, scale=True)

        collision_collection = self.get_or_create_collection('Collisions')
        collision_mesh = bpy.data.meshes.new(ob.name+"_collision_mesh")

        mesh = ob.data
        bm = bmesh.new()

        # convert the current mesh to a bmesh (must be in edit mode)
        # bpy.ops.object.mode_set(mode='EDIT')
        # bm.from_mesh(mesh)
        # bpy.ops.object.mode_set(mode='OBJECT')  # return to object mode

        dim_div_2 = ob.dimensions/2

        bm.verts.new(dim_div_2*Vector((-1, -1, -1)) +
                     ob.matrix_world.to_translation())
        bm.verts.new(dim_div_2*Vector((-1, -1, 1)) +
                     ob.matrix_world.to_translation())
        bm.verts.new(dim_div_2*Vector((-1, 1, 1)) +
                     ob.matrix_world.to_translation())
        bm.verts.new(dim_div_2*Vector((-1, 1, -1)) +
                     ob.matrix_world.to_translation())
        bm.verts.new(dim_div_2*Vector((1, -1, -1)) +
                     ob.matrix_world.to_translation())
        bm.verts.new(dim_div_2*Vector((1, -1, 1)) +
                     ob.matrix_world.to_translation())
        bm.verts.new(dim_div_2*Vector((1, 1, 1)) +
                     ob.matrix_world.to_translation())
        bm.verts.new(dim_div_2*Vector((1, 1, -1)) +
                     ob.matrix_world.to_translation())

        bm.verts.ensure_lookup_table()
        # edges = [(e.vertices.data.vertices[0], e.vertices.data.vertices[1])
        #          for e in ob.data.edges]

        faces = [[0, 1, 2, 3], [4, 5, 1, 0], [2, 1, 5, 6],
                 [0, 3, 7, 4], [3, 2, 6, 7], [7, 6, 5, 4]]

        # for f in ob.data.polygons:
        #     new_face = []
        #     for idx in f.vertices:
        #         new_face.append(vertices.index(tuple(ob.data.vertices[idx].co)))
        #     faces.append(tuple(new_face))

        # print("vertices",vertices,"edges",edges,"faces",faces)

        # bm.from_mesh(collision_mesh)
        # bmverts = []
        for face in faces:
            bmface = []
            # print(face,list(bm.verts))
            bmface.append(bm.verts[face[0]])
            bmface.append(bm.verts[face[1]])
            bmface.append(bm.verts[face[2]])
            bmface.append(bm.verts[face[3]])
            bm.faces.new(bmface)
            # bmverts.append(tuple(bmface))
        # bmverts = [bm.verts.new(co) for co in vertices]
        # bm.vertices.new(bmverts)
        bm.faces.ensure_lookup_table()

        bm.to_mesh(collision_mesh)
        bm.free()

        collision_object = collision_collection.objects.get(
            ob.name+"_collision")
        if collision_object != None:
            bpy.data.objects.remove(collision_object, do_unlink=True)

        collision_object = bpy.data.objects.new(
            ob.name+"_collision", collision_mesh)
        collision_collection.objects.link(collision_object)

        # collision_mesh.from_pydata(vertices, edges, faces)
        # collision_mesh.update()

        # bpy.ops.wm.properties_edit(data_path="object", property_name="physics", property_type='STRING', is_overridable_library=False, description="", use_soft_limits=False, default_float=(1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1), min_float=0, max_float=1, soft_min_float=0, soft_max_float=1, precision=3, step_float=1, subtype='NONE', default_string="true", eval_string="1.0")

        collision_object["data"] = "physics"
        collision_object["type"] = "box"

        bpy.ops.object.select_all(action='DESELECT')
        collision_object.select_set(True)
        bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='BOUNDS')
        

        # collision_object.hide_set(True)

        self.report({"INFO"}, "Added box physics")

        return {'FINISHED'}
